Package: linprog
Version: 0.9-6
Date: 2026-01-19
Title: Linear Programming / Optimization
Authors@R: person(given = "Arne", family = "Henningsen",
   role = c("aut", "cre"), email = "arne.henningsen@gmail.com")
Author: Arne Henningsen [aut, cre]
Maintainer: Arne Henningsen <arne.henningsen@gmail.com>
Depends: R (>= 2.4.0), lpSolve
Description: Can be used to solve Linear Programming / Linear
   Optimization problems by using the simplex algorithm.
License: GPL (>= 2)
URL: http://linprog.r-forge.r-project.org/
NeedsCompilation: no
Packaged: 2026-01-19 13:31:18 UTC; gsl324
Repository: CRAN
Date/Publication: 2026-01-19 14:10:02 UTC
